var child_process = require('child_process');
var camelcase = require("camelcase");
var awscli = {};
var logd = function() {};
awscli.setDebug = function() {
    logd = function(s) { console.log(s); };
};
awscli.exec = function(service, command, opts, callback) {
    var cmdlin = ["aws ", service, command];
    opts = opts || {};
    Object.keys(opts).forEach(function(opt) {
        if(opt !== "") {
            cmdlin.push(opt);
        }
        cmdlin.push(opts[opt]);
    });
    logd(cmdlin.join(' '));
    child_process.exec(cmdlin.join(' '), function(err, stdout, stderr) {
        if(stderr !== "") {
            console.error(stderr);
        }
        if(err) {
            console.error(err);
        }
        var json = null;
        try {
            if(stdout) {
                json = JSON.parse(stdout);
            } else {
                json = "";
            }
        } catch(ex) {
            console.error("EXCEPTION at JSON.parse:", ex.toString());
            console.error("stdout:", stdout);
            process.exit(1);
        }
        if(json != null) {
            callback(err, json);
        }
    });
};
(function() {
    "use strict";

    var AWS = require("aws-sdk");

    //
    // Get default region from the file ~/.aws/config
    //
    var aws_configure = require("./aws-configure");
    var region = aws_configure.getConfiguredRegion(aws_configure.getAwsProfile());
    if(region) {
        AWS.config.update({"region" : region });
    } else {
        console.error("Error: No AWS region name is determined");
        console.error("Set default region using an AWS CLI command `aws configure`.");
        process.exit(1);
    }

    //
    // Get AWS service class instance
    //
    awscli.getService = function (serviceName) {
        if(!(serviceName in AWS)) {
            console.error("ERROR: Invalid AWS service name", serviceName);
            process.exit(1);
        }
        var service = new AWS[serviceName]();
        if(!service) {
            console.error("ERROR: Fail to create AWS service", serviceName);
            process.exit(1);
        }
        return service;
    };

    // Declare the AWS CLI commands
    var services = {
        "dynamodb" : {
            "description" : "",
            "commands" : {
                "describe-table" : {
                    "description" : "",
                    "options" : [
                        { "name" : "table-name", "required" : true }
                    ]
                },
                "list-tables" : {
                    "description" : "",
                    "options" : [
                    ]
                },
                "scan" : {
                    "description" : "",
                    "options" : [
                        { "name" : "table-name", "required" : true },
                        { "name" : "max-items", "required" : true },
                        { "name" : "starting-token", "required" : false },
                        { "name" : "", "extra-option" : true }
                        /**
                        { "name" : "index-name", "required" : false },
                        { "legacy": true, "name" : "attributes-to-get", "required" : false },
                        { "name" : "select", "required" : false },
                        { "deprecated": true, "name" : "scan-filter", "required" : false },
                        { "name" : "conditional-operator", "required" : false },
                        { "name" : "return-consumed-capacity", "required" : false },
                        { "name" : "total-segments", "required" : false },
                        { "name" : "segment", "required" : false },
                        { "name" : "projection-expression", "required" : false },
                        { "name" : "filter-expression", "required" : false },
                        { "name" : "expression-attribute-names", "required" : false },
                        { "name" : "expression-attribute-values", "required" : false },
                        { "name" : "consistent-read", "no-arg" : true, "required" : false },
                        { "name" : "page-size", "required" : false }
                        }
                        **/
                    ]
                },
                "query" : {
                    "description" : "",
                    "options" : [
                        { "name" : "table-name", "required" : true },
                        { "name" : "max-items", "required" : true },
                        { "name" : "starting-token", "required" : false },
                        { "name" : "", "extra-option" : true }
                        /****
                        { "name" : "index-name", "required" : false },
                        { "name" : "select", "required" : false },
                        { "legacy": true, "name" : "attributes-to-get", "required" : false },
                        { "name" : "consistent-read", "no-arg" : true, "required" : false },
                        { "name" : "key-conditions", "required" : false },
                        { "legacy": true, "name" : "query-filter", "required" : false },
                        { "legacy": true, "name" : "conditional-operator", "required" : false },
                        { "name" : "scan-index-forward", "no-arg": true, "required" : false },
                        { "name" : "return-consumed-capacity", "required" : false },
                        { "name" : "projection-expression", "required" : false },
                        { "name" : "filter-expression", "required" : false }
                        ****/
                    ]
                }
            }
        },
        "iot" : {
            "description" : "",
            "commands": {
                "attach-principal-policy" : {
                    "description" : "",
                    "options" : [
                        { "name" : "policy-name", "required" : true },
                        { "name" : "principal", "required" : true }
                    ]
                },
                "attach-thing-principal" : {
                    "description" : "",
                    "options" : [
                        { "name" : "thing-name", "required" : true },
                        { "name" : "principal", "required" : true }
                    ]
                },
                "create-keys-and-certificate" : {
                    "description" : "",
                    "options" : [
                        { "name" : "set-as-active", "no-arg": true }
                    ]
                },
                "create-policy" : {
                    "description" : "",
                    "options" : [
                        { "name" : "policy-name", "required" : true },
                        { "name" : "policy-document", "required" : true }
                    ]
                },
                "create-thing" : {
                    "description" : "",
                    "options" : [
                        { "name" : "thing-name", "required" : true },
                        { "name" : "attribute-payload" }
                    ]
                },
                "describe-certificate" : {
                    "options" : [
                        { "name" : "certificate-id", "required" : true }
                    ]
                },
                "describe-thing" : {
                    "options" : [
                        { "name" : "thing-name", "required" : true }
                    ]
                },
                "delete-thing" : {
                    "options" : [
                        { "name" : "thing-name", "required" : true }
                    ]
                },
                "list-certificates" : {
                    "options" : [
                    ]
                },
                "list-principal-policies" : {
                    "options" : [
                        { "name" : "principal", "required" : true }
                    ]
                },
                "list-principal-things" : {
                    "options" : [
                        { "name" : "principal", "required" : true }
                    ]
                },
                "list-thing-principals" : {
                    "options" : [
                        { "name" : "thing-name", "required" : true }
                    ]
                },
                "list-topic-rules" : {
                    "options" : [
                    ]
                },
                "list-things" : {
                    "options" : [
                    ]
                },
                "get-policy" : {
                    "options" : [
                        { "name" : "policy-name", "required" : true }
                    ]
                },
                "get-topic-rule" : {
                    "options" : [
                        { "name" : "rule-name", "required" : true }
                    ]
                },
                "list-policies" : {
                    "options" : [
                    ]
                }
            }
        }
    };
    var callbackError = function(message, callback) {
        if(callback) {
            callback.call(null, error, null);
        }
    };

    //
    // Create function to call the command through AWS CLI
    //
    Object.keys(services).forEach(function (service_name) {
        var service_def = services[service_name];
        var service = {};
        Object.keys(service_def.commands).forEach(function(command_name) {
            var function_name = camelcase(command_name);
            service[function_name] = (function(command_options) { return function() {
                var args = Array.apply(null, arguments);
                var callback = null;
                if(args.length > 0) {
                    if(typeof(args[args.length - 1]) === typeof(function(){})) {
                        callback = args.splice(-1, 1)[0];
                    }
                }
                var options = {};
                for(var optIdx = 0; optIdx < command_options.length; optIdx++) {
                    var option = command_options[optIdx];
                    if(optIdx < args.length) {
                        if(args[optIdx] !== null) {
                            if("no-arg" in option && option["no-arg"]) {
                                if(args[optIdx]) {
                                    options["--" + option.name] = "";
                                }
                            } else if("extra-option" in option && option["extra-option"]) {
                                Object.keys(args[optIdx]).forEach(function(key) {
                                    if(args[optIdx][key] == null) {
                                        options["--" + key] = "";
                                    } else {
                                        options["--" + key] = args[optIdx][key];
                                    }
                                });
                            } else {
                                options["--" + option.name] = args[optIdx];
                            }
                        }
                    } else if(option.required) {
                        callbackError("ERROR aws." + svc + "." + function_name + ":"
                            + option.name + "is required");
                        return;
                    }
                }
                awscli.exec(service_name, command_name, options, callback);
            };}(service_def.commands[command_name].options));
        });
        awscli[service_name] = service;
    });

    //
    // Convert an object to string for the command line option value
    //
    awscli.jsonAsQuotedString = function(obj) {
        return '"' + JSON.stringify(obj).replace(/"/g, '\\"') + '"';
    };
}());
module.exports = awscli;
